/*
 * com.simulogic.cese -- Cell Electrophysiology Simulation Environment (CESE)
 * 
 * Copyright (C) 1999 - 2005 Sergey Missan
 * 
 * Dalhousie Univ., Halifax, NS, Canada
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 *  
 */

package com.simulogic.cese;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Frame;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.URL;
import java.net.URLConnection;
import java.util.prefs.Preferences;

import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.text.Document;
import javax.swing.text.html.FormSubmitEvent;
import javax.swing.text.html.HTMLEditorKit;

/**
 * About action implementation
 */
public class IntroWindow extends JFrame implements HyperlinkListener {

	private JEditorPane browserPane = null;
	private String startURL = "http://www.simulogic.com/intro.html";
	private String goURL = "http://www.simulogic.com/cese_survey.jsp";

	/**
	 * A reference to Preferences object
	 */
	private Preferences introPreferences = null;

	private String ver = "";

	/**
	 * Default constructor
	 * 
	 * @param f
	 *            parent frame
	 */
	public IntroWindow(Frame f) {
		super();

		introPreferences = Preferences.userNodeForPackage(this.getClass());
		ver = introPreferences.get("ver", "");

		if (!ver.equalsIgnoreCase(UserHome.CESE_VERSION_MINOR)) {
			getContentPane().add(createHtmlPane(), BorderLayout.CENTER);

			pack();
			setVisible(true);
		}
	}

	private JScrollPane createHtmlPane() {
		JScrollPane scrollPane = null;

		HTMLEditorKit ek = new HTMLEditorKit();
		ek.setAutoFormSubmission(false);

		browserPane = new JEditorPane();
		browserPane.setContentType("text/html");
		browserPane.setEditorKit(ek);

		browserPane.addHyperlinkListener(this);
		browserPane.setEditable(false);

		browserPane.setText("<html><body>Downloading...</body></html>");

		scrollPane = new JScrollPane(browserPane);
		scrollPane.setPreferredSize(new Dimension(450, 550));

		com.simulogic.utils.SwingWorker worker = new com.simulogic.utils.SwingWorker() {

			public Object construct() {
				try {
					browserPane.setPage(startURL);
				} catch (Exception e) {
				}
				return ResourceManager.getResource("Done");
			}

			public void finished() {
				browserPane
						.addPropertyChangeListener(new PropertyChangeListener() {
							public void propertyChange(PropertyChangeEvent e) {
								if (e.getPropertyName().equals("page")) {
									setTitle(browserPane
											.getDocument()
											.getProperty(Document.TitleProperty)
											.toString());
								}
							}
						});
			}
		};

		worker.start();

		return scrollPane;
	}

	public void hyperlinkUpdate(HyperlinkEvent event) {

		if (event instanceof FormSubmitEvent) {
			try {
				URLConnection conn = new URL(goURL).openConnection();
				conn.setDoOutput(true);
				OutputStreamWriter wr = new OutputStreamWriter(conn
						.getOutputStream());
				wr.write(((FormSubmitEvent) event).getData());
				wr.flush();

				BufferedReader rd = new BufferedReader(new InputStreamReader(
						conn.getInputStream()));

				String line;

				while ((line = rd.readLine()) != null) {

				}

				wr.close();
				rd.close();
			} catch (Exception e) {
			}

			introPreferences.put("ver", UserHome.CESE_VERSION_MINOR);
			this.setVisible(false);
			this.dispose();
		}

		if (event.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
			try {
				browserPane.setPage(event.getURL());
			} catch (IOException ioe) {
			}
		}

	}
}
