/*
 * com.simulogic.cese -- Cell Electrophysiology Simulation Environment (CESE)
 * 
 * Copyright (C) 1999 - 2005 Sergey Missan
 * 
 * Dalhousie Univ., Halifax, NS, Canada
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 *  
 */

package com.simulogic.cese.actions;

import java.awt.event.ActionEvent;
import java.io.File;

import javax.swing.JFileChooser;

import com.simulogic.cese.CESE;
import com.simulogic.cese.ModelManager;
import com.simulogic.cese.ResourceManager;
import com.simulogic.cese.UserHome;
import com.simulogic.utils.ActionSupport;
import com.simulogic.utils.ExceptionDialog;
import com.simulogic.utils.ExtensionFileFilter;

/**
 * OpenRec action implementation
 */
public class OpenRecAction extends ActionSupport {

    private File dir, file;

    private ModelManager manager;

    /** 
     * Creates new OpenRecAction  
     *  
     * @param m model manager
     */
    public OpenRecAction(ModelManager m) {
        this();
        manager = m;
    }

    /** Creates new OpenRecAction */
    public OpenRecAction() {
        super(ResourceManager.getResource("Open_Record..."), ResourceManager
                .getResource("Open_record"), ResourceManager
                .getResource("open_rec_image"));
     }

    public void actionPerformed(ActionEvent evt) {
    	if (dir == null)
			dir = new File(UserHome.CESE_HOME + UserHome.FS
					+ ResourceManager.getResource("recs_dir"));

        try {
            JFileChooser filechooser = new JFileChooser(dir);
            filechooser.setDialogTitle(ResourceManager
                    .getResource("Load_Record_Parameters"));
            ExtensionFileFilter preferredFilter = new ExtensionFileFilter(
                    ResourceManager.getResource("rec_extension"),
                    ResourceManager.getResource("Record_Parameters_Files"));
            filechooser.addChoosableFileFilter(preferredFilter);
            filechooser.setFileFilter(preferredFilter);

            int retVal = filechooser.showOpenDialog(CESE.getEnvironment());
            if (retVal == JFileChooser.APPROVE_OPTION) {
                file = filechooser.getSelectedFile();
                dir = filechooser.getCurrentDirectory();
                if (file.getPath() == null) return;
				manager.getRecordBuilder().getRecentRecords().addFileName(file.getPath());
                manager.getRecordBuilder().setRecFile(file);
            }
        } catch (Exception e) {
            ExceptionDialog.showExceptionDialog(ResourceManager
                    .getResource("Error"), ResourceManager
                    .getResource("Error_loading_record_file."), e);
        } finally {
        }
    }
}