/*
 * com.simulogic.dataview -- The Java2D Plotting Framework
 *
 * Copyright (C) 1999 - 2005 Sergey Missan
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.simulogic.dataview.export;

import java.io.IOException;

import com.simulogic.dataview.DataChannel;
import com.simulogic.dataview.DataStorage;
import com.simulogic.dataview.ResourceManager;
import com.simulogic.dataview.ViewContent;

/**
 * Class that exports data to ATF format
 */
public class AtfExport {

	private com.simulogic.dataview.ViewContent view;

	/**
	 * Default constructor
	 * 
	 * @param v
	 *            view content
	 */
	public AtfExport(ViewContent v) {
		view = v;
	}

	/**
	 * Exports the content of <code>DataStorage</code> to ATF file
	 * 
	 * @param fileName
	 *            name of the file to export data
	 * @throws IOException
	 *             if i/o problem occurs
	 */
	public synchronized void export(String fileName) throws java.io.IOException {
		java.io.PrintWriter datExp = new java.io.PrintWriter(
				new java.io.FileWriter(fileName));

		AsciiExport asc = new AsciiExport(view);

		if ((view.getTabs().indexOfTab(ResourceManager.getResource("Cursors")) == view
				.getTabs().getSelectedIndex())
				&& (view.getTabs().indexOfTab(
						ResourceManager.getResource("Cursors")) != -1)) {
			datExp.print(buildATFHeader(view.getView().getCursors()
					.getCursorsStorage()));

			datExp.print(asc.exportDataASCII(view.getView().getCursors()
					.getCursorsStorage()));
		} else {
			datExp.print(buildATFHeader(view.getView().getStorage()));

			datExp.print(asc.exportDataASCII(view.getView().getStorage()));
		}

		datExp.flush();
		datExp.close();
	}

	private String buildATFHeader(DataStorage storage) {
		DataChannel channel = null;
		StringBuffer hdr = new StringBuffer();
		int chans = 0;

		try {
			for (int i = 0; i < storage.getGroupsSize(); i++) {
				for (int j = 0; j < storage.getChannelsSize(i); j++) {
					channel = storage.getChannel(i, j);
					if (!channel.getAttribute().isHidden())
						chans++;
				}
			}
			hdr.append("ATF" + '\t' + "1.0");
			hdr.append('\n');
			hdr.append(1 + '\t' + chans);
			hdr.append('\n');
			hdr.append("\"Comment=" + view.getComment() + "\"");
			hdr.append('\n');
		} catch (Exception e) {
		}

		return hdr.toString();
	}

}
